#ifndef TIMELINEITEM_H
#define TIMELINEITEM_H

#include <QObject>
#include <QMediaPlayer>
#include <QVideoProbe>
#include <QFile>
#include <QBuffer>

class TimelineItem : public QObject
{
    Q_OBJECT

private:
    qreal m_offset = 0;
    qreal m_duration = 0;
    bool m_selected = false;

    bool m_visible = true;
    bool m_locked = false;
    bool m_baseLayer = false;

    QMediaPlayer* m_mediaPlayer;

    QString m_name;

    QUrl m_filepath;

    qreal m_tempTranslation = 0;

    QScopedPointer<QByteArray> m_videoByteArray;
    QScopedPointer<QBuffer> m_videoBuffer;

public:
    explicit TimelineItem(QObject *parent = nullptr);
    ~TimelineItem();

    QMediaPlayer* mediaPlayer() { return m_mediaPlayer; }

    void loadVideo(QString path);

    QString name() { return m_name; }


    bool visible() { return m_visible; }

    bool locked() { return m_locked || isBaseLayer(); }

    bool isBaseLayer() { return m_baseLayer; }


    qreal offset() { return m_offset; }
    qreal displayOffset() { return m_offset + m_tempTranslation; }

    qreal duration() { return (qreal)mediaPlayer()->duration() / 1000.0; }

    bool selected() { return m_selected; }


    qreal temporaryTranslation() { return m_tempTranslation; }
    void setTemporaryTranslation(qreal translation) { m_tempTranslation = translation; }
    void commitTemporaryTranslation()
    {
        setOffset(m_offset + m_tempTranslation);
        setTemporaryTranslation(0);
    }
    void revertTemporaryTranslation()
    {
        setTemporaryTranslation(0);
    }

    const QUrl& url() { return m_filepath; }


signals:
    void nameChanged(QString name);
    void visibilityChanged(bool visible);
    void lockedChanged(bool visible);
    void baseLayerChanged(bool isBaseLayer);
    void offsetChanged(qreal offset);
    void selectedChanged(bool selected);

public slots:
    void setName(QString name)
    {
        m_name = name;
        emit nameChanged(m_name);
    }

    void setPosition(qint64 position)
    {
        auto value = position - (qint64)(offset() * 1000);
        m_mediaPlayer->setPosition(value);
    }

    void setVisible(bool visible)
    {
        m_visible = visible;
        emit visibilityChanged(m_visible);
    }

    void setLocked(bool locked)
    {
        m_locked = locked;
        emit lockedChanged(m_locked);
    }

    void setBaseLayer(bool baseLayer)
    {
        m_baseLayer = baseLayer;
        emit baseLayerChanged(m_baseLayer);
    }

    void setOffset(qreal position)
    {
        m_offset = position;
        emit offsetChanged(m_offset);
    }

    void setSelected(bool selected)
    {
        m_selected = selected;
        emit selectedChanged(m_selected);
    }
};

#endif // TIMELINEITEM_H
