#include "rmcplot.h"
#include <QDebug>
#include "Utilities/graphicsmath.h"

RmcPlot::RmcPlot(QObject *parent) : QObject(parent)
{

}

qreal RmcPlot::relativeValueForQuantityAtTime(RmcPlotQuantity* quantity, qreal time)
{
    qreal value = valueForQuantityAtTime(quantity, time);

    auto scaleGroup = scaleGroups().value(quantity->scaleGroup());
    qreal bottomScale = scaleGroup->bottomScale();
    qreal topScale = scaleGroup->topScale();
    qreal invLerped = invLerp(bottomScale, topScale, value);
    return invLerped;
}

qreal RmcPlot::valueForQuantityAtTime(RmcPlotQuantity* quantity, qreal time)
{
    // Figure out what item this quantity corresponds to
    int index = -1;
    for (int i = 0; i < items().size(); i++)
    {
        auto item = items().at(i);

        auto itemAddress = item->address();
        auto quantityFormula = quantity->formula();
        if (itemAddress == quantityFormula)
        {
            index = i;
            break;
        }
    }
    if (index == -1) return FLT_MIN;

    // Figure out what sample point the time corresponds to
    int sampleTime = time / samplePeriod() + samples().keys().at(0);

    if (samples().contains(sampleTime))
    {
        return samples().value(sampleTime)->valueAt(index);
    }
    else
    {
        // Case of playhead before start of plot
        if (sampleTime < samples().firstKey())
        {
            if (samples().first() == nullptr)
            {
                return 0;
            }
            return samples().first()->valueAt(index);
        }

        // Case of playhead after end of plot
        if (sampleTime > samples().lastKey())
        {
            if (samples().last() == nullptr)
            {
                return 0;
            }
            return samples().last()->valueAt(index);
        }

        // Let's find the nearest left or right values
        auto keys = samples().keys();
        int i;
        for (i = 0; keys.count() < i && keys.at(i) < sampleTime; i++);
        int leftSampleTime = keys.count() < i ? keys.at(i) : keys.at(keys.count() - 1);

        // Currently just giving the left sample time - allow for interpolation options?
        return samples().value(leftSampleTime)->valueAt(index);
    }
}
