#ifndef RMCPLOT_H
#define RMCPLOT_H

#include "rmcplotquantity.h"
#include "rmcscalegroup.h"
#include "rmcplotsample.h"
#include "rmcplotitem.h"
#include <QObject>
#include <QMap>
#include <QHash>

class RmcPlot : public QObject
{
    Q_OBJECT
public:
    explicit RmcPlot(QObject *parent = nullptr);

    enum class ControllerType {
        RMC70,
        RMC150,
        RMC200
    };

    QString name() { return m_name; }
    void setName(QString name) { m_name = name; }

    qreal samplePeriod() { return m_samplePeriod; }
    void setSamplePeriod(qreal period) { m_samplePeriod = period; }

    qreal duration()
    {
        return (qreal)(samples().lastKey() - samples().firstKey()) * samplePeriod();
    }

    ControllerType controllerType() { return m_controllerType; }
    void setControllerType(ControllerType type) { m_controllerType = type; }


    const QList<RmcPlotQuantity*>& quantities() { return m_quantities; }
    void appendQuantity(RmcPlotQuantity* quantity)
    {
        quantity->setParent(this);
        m_quantities.append(quantity);
    }

    const QList<RmcPlotItem*>& items() { return m_items; }
    void appendItem(RmcPlotItem* item)
    {
        item->setParent(this);
        m_items.append(item);
    }

    const QMap<int, RmcPlotSample*>& samples() { return m_samples; }
    void appendSample(int interval, RmcPlotSample* sample)
    {
        sample->setParent(this);
        m_samples.insert(interval, sample);
    }

    const QHash<int, RmcScaleGroup*>& scaleGroups() { return m_scaleGroups; }
    RmcScaleGroup* scaleGroup(int id) { return m_scaleGroups.value(id); }
    void appendScaleGroup(int id, RmcScaleGroup* scaleGroup)
    {
        scaleGroup->setParent(this);
        m_scaleGroups.insert(id, scaleGroup);
    }

    qreal relativeValueForQuantityAtTime(RmcPlotQuantity* quantity, qreal time);
    qreal valueForQuantityAtTime(RmcPlotQuantity* quantity, qreal time);

private:
    QString m_name;
    qreal m_samplePeriod = 0.001;
    ControllerType m_controllerType;
    QList<RmcPlotQuantity*> m_quantities;
    QList<RmcPlotItem*> m_items;
    QMap<int, RmcPlotSample*> m_samples;
    QHash<int, RmcScaleGroup*> m_scaleGroups;

    // TODO: When the user highlights a quantity in the QuantityPanelView, go to the Engine
    // and set the active plot's selected quantity to this.
    // - In Engine, go to m_plot's active plot (might be hardcoded to the first one?)
    //   and set its selected quantity to whatever was in the QuantityPanelItem
    // - Line 781 of rmcplotfile.cpp - it's indeed hardcoded to do the first plot!


signals:
    void visibilityChanged();
};

#endif // RMCPLOT_H
