#ifndef RMCPLOTFILE_H
#define RMCPLOTFILE_H

#include <QObject>
#include <QThread>
#include <QXmlStreamReader>
#include <QFile>
#include <QDebug>
#include <QImage>
#include <QPainter>

#include <QRegularExpression>
#include <QRegularExpressionMatch>

#include "rmcplotitem.h"
#include "rmcplotquantity.h"
#include "rmcplot.h"
#include "rmcscalegroup.h"
#include "rmcplotpixelcolumn.h"
#include "Utilities/graphicsmath.h"

#include "zlib.h"

typedef QPair<RmcPlotQuantity*, QList<RmcPlotPixelColumn>*> QuantityValuePair;

class RmcPlotFile : public QThread
{
    Q_OBJECT
    void run();
    void onFinished();

    enum class QuantityType {
        Position = 100,
        Velocity = 101,
        Acceleration = 102,
        Counts = 103,
        RawCounts = 104,
        PressureForce = 105,
        PressureForceRate = 106,
        SecCounts = 107,
        SecRawCounts = 109,
        Gains = 110,
        Jerk = 111,
        ControlOutput75_150 = 112,
        ControlOutput200 = 110,
        Other = -1
    };


public:
    explicit RmcPlotFile(QObject *parent = nullptr);

    void loadPlotFile(const QString& path);

    QString name() { return m_name; }

    const QUrl& url() { return m_url; }

    QImage* getTYImage(qreal width, qreal height, int plotIndex);
    QImage* getXYImage(qreal width, qreal height, int plotIndex, int xQuantityIndex);

    const QList<RmcPlot*>& plots() { return m_plots; }

private:
    void parseUncompressedPlot(QFile& file);
    void parseCompressedPlot(QFile& file);
    void parseNextElement(QXmlStreamReader* reader);

    void handlePlotElement(QXmlStreamReader *reader);
    void handleControllerElement(QXmlStreamReader *reader);
    void handleItemElement(QXmlStreamReader *reader);
    void handleQuantityElement(QXmlStreamReader *reader);
    void handleSampleElement(QXmlStreamReader *reader);

    QuantityType quantityTypeForQuantity(QString *name, QString *formula);

    bool parseBool(const QString *attribute, bool *destination, bool defaultValue, QXmlStreamAttributes *attributes);
    bool parseReal(const QString *attribute, qreal *destination, qreal defaultValue, QXmlStreamAttributes *attributes);
    bool parseInt(const QString *attribute, int *destination, int defaultValue, QXmlStreamAttributes *attributes);

    void updateScaleGroups(RmcPlot* data);

    void getQuantitiesAndValues(qreal width, qreal height, RmcPlot* data, QList<QuantityValuePair>* quantitiesAndValues);
    void getQuantitiesAndValuesXY(qreal width, qreal height, RmcPlot* data, QList<QuantityValuePair>* quantitiesAndValues, int xQuantityIndex);

    void populateQuantityPixelColumns(RmcPlot* data, QList<QPair<qreal, qreal>>* groups, int firstInt, int intRange, qreal width, qreal height, QList<QuantityValuePair>* quantitiesAndValues);
    void populateQuantityPixelColumnsXY(RmcPlot* data, QList<QPair<qreal, qreal>>* groups, int firstInt, int intRange, qreal width, qreal height, RmcPlotQuantity* xQuantity, QList<QuantityValuePair>* quantitiesAndValues);

    QImage* renderImage(qreal width, qreal height, QList<QuantityValuePair>* quantitiesAndValues);
    QImage* renderXYImage(qreal width, qreal height, QList<QuantityValuePair>* quantitiesAndValues);

    void paintQuantity(QuantityValuePair* quantityAndColumns, QPainter* painter);

    QString m_errorString;

    QString m_name;
    QUrl m_url;
    QList<RmcPlot*> m_plots;
    int m_autogeneratedScaleGroup = 1000;

    static const QString filter_str, graph_str, autoScale_str, centerOn0_str, minRange_str,
    bottomScale_str, topScale_str, autoGroup_str, scaleGroup_str, railToRail_str;

signals:
    void loadedSuccessfully();
    void progressLoading(int pos, int size);
    void errorOccurred(QString errorString);
};


#endif // RMCPLOTFILE_H
