#include "helpwindow.h"

HelpWindow::HelpWindow(QWidget *parent) : QWidget(parent, Qt::Window)
{
    setWindowTitle("VideoSync Help");
    setAttribute(Qt::WA_DeleteOnClose, true);

    QString m_collectionPath = QApplication::applicationDirPath() + "/help/help_collection.qhc";
    m_helpEngine = new QHelpEngine(m_collectionPath, this);
    connect(m_helpEngine->contentWidget(), &QHelpContentWidget::linkActivated,
            this, &HelpWindow::onLinkActivated);

    m_helpWebEngineView = new QWebEngineView;

    m_helpSplitter = new QSplitter(Qt::Orientation::Horizontal, this);
    m_helpSplitter->addWidget(m_helpEngine->contentWidget());
    m_helpSplitter->addWidget(m_helpWebEngineView);

    // Set up toolbar
    m_toolbar = new QToolBar(this);
    m_toolbar->setToolButtonStyle(Qt::ToolButtonStyle::ToolButtonTextUnderIcon);
    m_toolbar->addAction(m_helpWebEngineView->pageAction(QWebEnginePage::Back));
    m_toolbar->addAction(m_helpWebEngineView->pageAction(QWebEnginePage::Forward));

    m_layout = new QVBoxLayout(this);
    m_layout->setSpacing(0);
    m_layout->setContentsMargins(0,0,0,0);
    m_layout->addWidget(m_toolbar);
    m_layout->addWidget(m_helpSplitter);

    if (m_helpEngine->setupData())
    {
        onLinkActivated(QUrl("qthelp://deltamotion.com.videosync.1.0/doc/getting_started.html"));
    }
    else
    {
        QMessageBox::critical(this, "VideoSync", QString("The help documentation could not be loaded: %1").arg(m_helpEngine->error()));
        close();
    }
}

HelpWindow::~HelpWindow()
{
}

void HelpWindow::onLinkActivated(const QUrl& url)
{
    QUrl baseUrl = QUrl(QApplication::applicationDirPath() + "/help/");
    m_helpWebEngineView->setContent(
                m_helpEngine->fileData(url),
                "text/html;charset=UTF-8",
                baseUrl
                );
}
