#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QVBoxLayout>
#include <QDockWidget>
#include <QMenuBar>
#include <QSplitter>
#include <QMessageBox>
#include <QApplication>

#include <QtHelp>
#include <QWebEngineView>

#include "Utilities/Constants/colorconstants.h"

#include "Views/VideoView/videoview.h"
#include "Views/ControlView/controlview.h"
#include "Views/TimelineView/timelineview.h"
#include "Models/Engine/engine.h"
#include "Views/PropertiesPanel/propertiesview.h"
#include "Views/QuantitiesPanel/quantitypanelview.h"
#include "Views/XYPlot/xyplotpanelview.h"

#include "Views/HelpWindow/helpwindow.h"


class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    MainWindow(QWidget *parent = nullptr);

private:
    // Overall layout
    QWidget* m_overallWidget;
    QVBoxLayout* m_overallLayout;

    VideoView* m_videoView;
    ControlView* m_controlView;
    TimelineView* m_timelineView;

    // Splitter
    QVBoxLayout* m_upperLayout;
    QWidget* m_upperWidget;
    QSplitter* m_mainSplitter;

    // Properties view
    QDockWidget* m_propertiesDockWidget;
    PropertiesView* m_propertiesView;

    // Quantity panel view
    QDockWidget* m_quantityPanelDockWidget;
    QuantityPanelView* m_quantityPanelView;

    // XY plot panel view (probably placeholder)
    QDockWidget* m_xyPlotPanelDockWidget;
    XYPlotPanelView* m_xyPlotPanelView;

    // Menus
    QMenu* m_fileMenu;
    QMenu* m_viewMenu;
    QMenu* m_workspaceMenu;
    QMenu* m_windowMenu;
    QMenu* m_helpMenu;

    // File actions
    QAction* m_openAction;
    QAction* m_saveAction;
    QAction* m_loadVideoAction;
    QAction* m_loadBaseVideoAction;
    QAction* m_loadPlotAction;
    QAction* m_exitAction;

    // Workspace actions
    QAction* m_syncAction;
    QAction* m_analyzeAction;
    QActionGroup* m_workspaceActionGroup;

    // Help actions
    QAction* m_openHelpAction;
    QAction* m_submitBugReportAction;
    QAction* m_aboutVideoSyncAction;
    QAction* m_aboutQtAction;

    Engine* m_engine = Engine::engine;

private slots:
    void showImportVideoModal();
    void showImportBaseVideoModal();
    void showImportPlotModal();
    void showSaveAsModal();
    void showOpenModal();

    void showHelpWindow();
};
#endif // MAINWINDOW_H
