#include "propertiesview.h"

/*
 * TODO: allow for changing the index of the current plot from combo box
 */
PropertiesView::PropertiesView(QWidget *parent) : QWidget(parent)
{
    connect(Engine::engine, &Engine::timingsChanged,
            this, &PropertiesView::onSelectionChanged);

    QFont bold;
    bold.setBold(true);

    m_titleLabel = new QLabel("Video Title", this);
    m_titleLabel->setFont(bold);
    m_titleLabel->setWordWrap(true);

    m_durationLabel = new QLabel("", this);

    m_offsetBox = new QDoubleSpinBox(this);
    m_offsetBox->setMinimum(-999.0);
    m_offsetBox->setDecimals(3);
    m_offsetBox->setMaximum(999.0);
    connect(m_offsetBox, QOverload<qreal>::of(&QDoubleSpinBox::valueChanged),
            this, &PropertiesView::onUserChangedOffsetFromSpinBox);

    m_plotBox = new QComboBox(this);

    m_layout = new QFormLayout(this);
    m_layout->addRow(m_titleLabel);
    m_layout->addRow("Duration:", m_durationLabel);
    m_layout->addRow("Offset:", m_offsetBox);
    m_layout->addRow("Plot:", m_plotBox);

    connect(Engine::engine, &Engine::selectionChanged, this, &PropertiesView::updateSelectedItem);
    updateWithNothing();
}

void PropertiesView::updateSelectedItem()
{
    // First, let's find what we have selected.
    // See if a video clip is selected...
    for (int i = 0; i < Engine::engine->timelineItems().count(); i++)
    {
        auto item = Engine::engine->timelineItems().at(i);
        if (item->selected())
        {
            updateWithVideoItem(item);
            return;
        }
    }

    // Not a video clip... what about the plot?
    if (Engine::engine->plotSelected())
    {
        updateWithPlotItem();
        return;
    }

    // Nothing is selected
    updateWithNothing();
}

void PropertiesView::updateWithVideoItem(TimelineItem* item)
{
    m_titleLabel->setText(item->name());
    m_durationLabel->setText(QString::number(item->duration()));

    m_offsetBox->setEnabled(true);

    m_plotBox->setEnabled(false);
    m_plotBox->clear();
}

void PropertiesView::updateWithPlotItem()
{
    m_titleLabel->setText(Engine::engine->plotFile()->name());
    m_durationLabel->setText(QString::number(Engine::engine->currentPlotDuration()));

    m_offsetBox->setEnabled(true);

    // Put all of the plots into the combo box
    m_plotBox->setEnabled(true);
    m_plotBox->clear();
    for (int i = 0; i < Engine::engine->plotFile()->plots().count(); i++) {
        RmcPlot* plot = Engine::engine->plotFile()->plots().at(i);
        m_plotBox->addItem(plot->name());
    }
}

void PropertiesView::updateWithNothing()
{
    m_titleLabel->setText("");
    m_durationLabel->setText("---");
    m_offsetBox->setEnabled(false);
    m_plotBox->setEnabled(false);
    m_plotBox->clear();
}

void PropertiesView::onUserChangedOffsetFromSpinBox(double value)
{
    qDebug() << "offset set to" << value;
    // Get the selected items
    auto allItems = Engine::engine->timelineItems();

    // Find the first selected item and set its offset to value
    for (auto i : allItems)
    {
        if (i->selected())
        {
            i->setOffset(value);
            break;
        }
    }

    if (Engine::engine->plotSelected())
    {
        Engine::engine->setPlotOffset(value);
    }
}

void PropertiesView::onSelectionChanged()
{
    bool timelineItemSelected = false;
    auto allItems = Engine::engine->timelineItems();
    // Find the first selected item and set its offset to value
    for (auto i : allItems)
    {
        if (i->selected())
        {
            auto isReadOnly = i->isBaseLayer() || i->locked();
            m_offsetBox->setReadOnly(isReadOnly);
            m_offsetBox->setValue(i->offset());
            timelineItemSelected = true;
            break;
        }
    }

    // If we didn't select a timeline item... did we select the plot?
    if (!timelineItemSelected && Engine::engine->plotSelected())
    {
        m_offsetBox->setValue(Engine::engine->plotOffset());
    }
}
