#include "quantitypanelitem.h"

QuantityPanelItem::QuantityPanelItem(QTreeWidget *parent) : QObject(parent), QTreeWidgetItem(parent)
{
    setText(0, "Quantity Name");
    setText(1, "Value");

    connect(Engine::engine, &Engine::positionChanged, this, &QuantityPanelItem::positionChanged);

    updateColorIcon(QColor(255,0,0));
}

void QuantityPanelItem::setQuantity(RmcPlotQuantity* quantity)
{
    m_quantity = quantity;
    setText(0, m_quantity->name());
    updateColorIcon(m_quantity->color());

    // If this item isn't graphed by the plot, don't even offer to display it
    Qt::ItemFlags flags = Qt::ItemIsSelectable | Qt::ItemIsEnabled;
    if (m_quantity->graphQuantity()) flags |= Qt::ItemIsUserCheckable;
    setFlags(flags);

    // Check or uncheck depending on whether or not the item should be graphed
    if (m_quantity->graphQuantity())
    {
        setCheckState(0, m_quantity->userGraph() ? Qt::Checked : Qt::Unchecked);
    }
    else
    {
        setCheckState(0, Qt::Unchecked);
    }
}

void QuantityPanelItem::positionChanged()
{
    auto plot = Engine::engine->currentPlot();
    setText(1, QString::number(
                plot->valueForQuantityAtTime(quantity(), Engine::engine->currentTime() - Engine::engine->plotDisplayOffset())
                )
            );
}


void QuantityPanelItem::updateColorIcon(QColor c)
{
    if (m_icon != nullptr) delete m_icon;

    m_icon = createColorIcon(c);
    setIcon(0, QIcon(*m_icon));
}

QPixmap* QuantityPanelItem::createColorIcon(QColor c)
{
    qreal penWidth = 10;
    int size = 100;
    QMargins margins(penWidth, penWidth, penWidth, penWidth);
    QRect rect = QRect(0,0, size, size).marginsRemoved(margins);

    QPixmap* pixmap = new QPixmap(size, size);
    pixmap->fill(QColorConstants::Transparent);

    QPainter painter(pixmap);

    painter.setRenderHint(QPainter::RenderHint::Antialiasing, true);
    painter.setBrush(QBrush(c));
    painter.setPen(QPen(QColor(c).darker(140), penWidth));
    painter.drawEllipse(rect);
    return pixmap;
}
