#include "quantitypanelview.h"

QuantityPanelView::QuantityPanelView(QWidget *parent) : QTreeWidget(parent)
{
    setColumnCount(2);
    setRootIsDecorated(false);
    setHeaderLabels({ "Quantity", "Value" });

    connect(Engine::engine, &Engine::plotLoaded, this, &QuantityPanelView::updateQuantities);
    connect(this, &QTreeWidget::itemChanged, this, &QuantityPanelView::onItemChanged);
    connect(this, &QTreeWidget::currentItemChanged, this, &QuantityPanelView::onCurrentItemChanged);
}

void QuantityPanelView::updateQuantities()
{
    // remove all existing items
    clearAllItems();

    // add an item for each quantity and link them up
    int numberOfQuantities = Engine::engine->currentPlot()->quantities().count();

    for (int i = 0; i < numberOfQuantities; i++)
    {
        auto q = Engine::engine->currentPlot()->quantities().at(i);
        QuantityPanelItem* item = new QuantityPanelItem();
        item->setQuantity(q);
        addTopLevelItem(item);
        item->positionChanged();
    }
}

void QuantityPanelView::onItemChanged(QTreeWidgetItem* baseItem, int column)
{
    auto item = dynamic_cast<QuantityPanelItem*>(baseItem);
    if (column > 0 || item == nullptr || item->quantity() == nullptr || !(item->flags() & Qt::ItemIsUserCheckable))
        return;

    item->quantity()->setUserGraph(item->checkState(0) == Qt::Checked);
}

void QuantityPanelView::onCurrentItemChanged(QTreeWidgetItem *current, QTreeWidgetItem * /*previous*/)
{
    if (current == nullptr)
    {
        return;
    }

    auto currentQPI = dynamic_cast<QuantityPanelItem*>(current);
    if (currentQPI == nullptr)
    {
        qDebug() << "An item in QuantityPanelView couldn't be cast to QuantityPanelItem";
        return;
    }

    auto quantity = currentQPI->quantity();
    if (quantity == nullptr)
    {
        qDebug() << "The quantity of the current QuantityPanelItem was null";
        return;
    }

    Engine::engine->setPlotYQuantity(quantity);
    qDebug() << "Selected quantity is" << quantity->name();
}
