#include "singlevideoview.h"

SingleVideoView::SingleVideoView(QWidget *parent, TimelineItem *item) : QWidget(parent)
{
    m_timelineItem = item;

    m_layout = new QVBoxLayout(this);

    // Create video widget
    m_videoWidget = new QVideoWidget(this);

    // Video title label
    m_videoTitleLabel = new QLabel(this);
    m_videoTitleLabel->setAlignment(Qt::AlignHCenter);
    m_videoTitleLabel->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Maximum);

    // Video position label
    m_videoPositionLabel = new QLabel("-.---s", this);
    m_videoPositionLabel->setAlignment(Qt::AlignHCenter);
    m_videoPositionLabel->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Maximum);

    layout()->setContentsMargins(0,0,0,0);
    layout()->addWidget(m_videoWidget);
    layout()->addWidget(m_videoTitleLabel);
    layout()->addWidget(m_videoPositionLabel);

    if (item != nullptr)
    {
        m_videoTitleLabel->setText(item->name());
        item->mediaPlayer()->setVideoOutput(m_videoWidget);
        connect(item->mediaPlayer(), &QMediaPlayer::positionChanged, this, &SingleVideoView::updatePositionLabel);

        connect(item, &TimelineItem::visibilityChanged, this, &SingleVideoView::updateVisibility);

        connect(item, &TimelineItem::nameChanged, m_videoTitleLabel, &QLabel::setText);

        updateVisibility(item->visible());
    }


}

void SingleVideoView::updatePositionLabel(qint64 position)
{
    m_videoPositionLabel->setText(QString("%1s").arg((qreal)position / 1000.0));
}

void SingleVideoView::updateVisibility(bool visible)
{
    this->setVisible(visible);
}
