#include "Views/MainWindow/mainwindow.h"

#include <QApplication>
#include <QLocale>
#include <QTranslator>

/*
 * My intent with the Engine class is for it to contain
 * information about the state of one plot/video sync "file"
 * at a time. For right now, the entire application runs off of
 * a singleton instance of an Engine, but in the future, we may
 * be able to open multiple simultaneously.
 */
Engine* Engine::engine;

/* Define default colors
 * I'd prefer if these could be stored somewhere else (such as in
 * colorconstants.h), but that causes lots of errors...
 */
const QColor DefaultColorConstants::LinePrimary = QColor(160, 160, 160, 160);
const QColor DefaultColorConstants::LineSecondary = QColor(200, 200, 200, 160);
const QColor DefaultColorConstants::Text = QColor(20, 20, 20);
const QColor DefaultColorConstants::TextSecondary = QColor(80, 80, 80);

const QColor DefaultColorConstants::ScrollBarBackground = QColor(160, 160, 160);
const QColor DefaultColorConstants::ScrollBarHandle = QColor(210, 210, 210);
const QColor DefaultColorConstants::ScrollBarBody = QColor(255, 255, 255);
const QColor DefaultColorConstants::ScrollBarBodyOutline = QColor(160, 160, 160);
const QColor DefaultColorConstants::ScrollBarVerticalLines = QColor(180, 180, 180);

const QColor DefaultColorConstants::BaseVideo = QColor(100, 100, 100);
const QColor DefaultColorConstants::AdditionalVideo = QColor(45, 90, 170);
const QColor DefaultColorConstants::PlotItem = QColor(50, 130, 60);

int main(int argc, char *argv[])
{
    QApplication a(argc, argv);

    QTranslator translator;
    const QStringList uiLanguages = QLocale::system().uiLanguages();
    for (const QString &locale : uiLanguages) {
        const QString baseName = "VideoSyncCPP_" + QLocale(locale).name();
        if (translator.load(":/i18n/" + baseName)) {
            a.installTranslator(&translator);
            break;
        }
    }

    Engine::engine = new Engine;
    MainWindow w;
    w.show();
    return a.exec();
}
